﻿/*
 Post-Deployment Script Template							
--------------------------------------------------------------------------------------
 This file contains SQL statements that will be executed after the build script.	
 Use SQLCMD syntax to include a file in the post-deployment script.			
 Example:      :r .\myfile.sql								
 Use SQLCMD syntax to reference a variable in the post-deployment script.		
 Example:      :setvar TableName MyTable							
               SELECT * FROM [$(TableName)]					
--------------------------------------------------------------------------------------

Use CompareVersions to compare versions.
CompareVersions returns: 1 if the first parameter is greater; -1 if second parameter is greater; 0 if equal

X.X.X.X current version
Y.Y.Y.Y Version where the query should be applied
IF dbo.[CompareVersions]('7.1.3','7.1.2.2') = 1
	UPDATE/INSERT ...
*/

DECLARE @dbName VARCHAR(50) = (SELECT DB_NAME())
DECLARE @newVersion VARCHAR(50) = (SELECT TOP 1 type_version FROM msdb.dbo.sysdac_instances WHERE instance_name = @dbName)


IF (EXISTS (SELECT TOP 1 * FROM tempdb.INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'TempExtendedProperties'))
BEGIN
	DECLARE @StoreId varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'StoreId')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'StoreId') AND @StoreId IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'StoreId', @value = @StoreId;
	
	DECLARE @CashRegister varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'CashRegister')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'CashRegister') AND @CashRegister IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'CashRegister', @value = @CashRegister;

	DECLARE @DBVersion varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'DBVersion')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'DBVersion') AND @DBVersion IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'DBVersion', @value = @DBVersion;

	DECLARE @DBLevel varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'DBLevel')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'DBLevel') AND @DBLevel IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'DBLevel', @value = @DBLevel;

	DECLARE @IsStoreHQ varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'IsStoreHQ')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'IsStoreHQ') AND @IsStoreHQ IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'IsStoreHQ', @value = @IsStoreHQ;

	DECLARE @ClientName varchar(100) = (SELECT TOP 1 Value FROM tempdb.dbo.TempExtendedProperties WHERE Name = 'ClientName')
	IF NOT EXISTS (select * from sys.extended_properties where name = 'ClientName') AND @ClientName IS NOT NULL
		EXECUTE sp_addextendedproperty @name = N'ClientName', @value = @ClientName;

	DROP TABLE tempdb.dbo.TempExtendedProperties
END

-- Convert database, only for levels S and L (HQ not necessary since all triggers should be active)
DECLARE @DbLevelValue VARCHAR(100) = (SELECT CONVERT(VARCHAR(199), Value) FROM sys.extended_properties WHERE Name = 'DBLevel')
IF @DbLevelValue = 'S'
	EXEC SysConvertDbToStoreServerDb @EnableStockTriggers = 1
ELSE IF @DbLevelValue = 'L' OR @DbLevelValue = 'LH'
	EXEC SysConvertDbToStoreDb @DbLevelValue

-- Enable ApplicationLog viewer for all customers -- the category row does not exist on initializations of databases
IF EXISTS (SELECT * FROM SecuritySystemPermissionsCategories WHERE CategoryId = 3)
	INSERT INTO SecuritySystemPermissions (OID, Name, Description, CategoryId)
	SELECT NEWID(), 'navBarItemSyncLogViewer', 'Utility --> Sync Log Viewer', 3
	WHERE NOT EXISTS (
		SELECT 1
		FROM SecuritySystemPermissions
		WHERE Name = 'navBarItemSyncLogViewer'
	);

---- Update database Version
--EXECUTE sp_updateextendedproperty @name = N'DBVersion', @value = @newVersion --this does not work because the version is chenged after the post deplyment script

-- Finish the update
IF EXISTS (SELECT * FROM sys.extended_properties WHERE Name = 'UpdateDBStartTime')
	EXECUTE sp_dropextendedproperty @name = 'UpdateDBStartTime'


BEGIN TRAN

	ALTER TABLE MOVEMENTS DISABLE TRIGGER TR_Movements_MovementAggregates

	Update Movements 
	SET RowDiscountValue = ABS(ISNULL(TotalDiscountValue, 0) * Quantity)		
	WHERE 
		RowDiscountValue IS NULL
		AND ReceiptId IS NOT NULL
	
	ALTER TABLE MOVEMENTS ENABLE TRIGGER TR_Movements_MovementAggregates

COMMIT

Update FidelityRedemptionTypes set Type = 'DSC' WHERE Type IS NULL


--------- Update the VoucherGeneratorId in Fidelityredemptionrules table only when it's null -------------

UPDATE FRR
SET FRR.VoucherGeneratorId = FCT.VoucherGeneratorId
FROM FidelityRedemptionRules FRR
JOIN FidelityRedemptionTypes FRT ON FRT.FidelityRedemptionTypeId = FRR.FidelityRedemptionTypeId
JOIN FidelityCardTypes FCT ON FCT.FidelityRedemptionTypeId = FRT.FidelityRedemptionTypeId
WHERE FRR.VoucherGeneratorId IS NULL AND FCT.VoucherGeneratorId IS NOT NULL

---------Insert the two types of distributions if they don't already exist-------------

INSERT INTO Distributions (Code, Name, CreateTime, TypeCode)
SELECT '001', 'Distribution by Family', GETDATE(), 'PF'
WHERE NOT EXISTS (SELECT 1 FROM Distributions WHERE TypeCode = 'PF')
UNION ALL
SELECT '002', 'General Distribution', GETDATE(), 'FR'
WHERE NOT EXISTS (SELECT 1 FROM Distributions WHERE TypeCode = 'FR');


---------Insert new Default Service Row Type inside ItemTemplateDetails if not exists--------
INSERT INTO ItemTemplateRowTypes (RowType, Name, Criteria)
SELECT 'SRD', 'Servizi Default', 'ProductId.ProductSubfamilyId.ProductFamilyId.ProductFamilyId = 13'
WHERE NOT EXISTS (SELECT 1 FROM ItemTemplateRowTypes WHERE RowType = 'SRD')
---------Insert new AccountantKey for sales of products for POS Terminals--------

INSERT INTO AccountantKeys
(
	[Code], [Description], [Value], [AccountTypeId], [AccountantKeyCategoryId], [TypeCode], [ShowInStoreDayClosure], [CauseIndex]
)
SELECT
	'CC_PT_VENDITAPRODOTTI', 'PT Vendita Prodotti', '', NULL, 0, 'CAU', NULL, NULL
WHERE NOT EXISTS (SELECT 1 FROM AccountantKeys WHERE CODE = 'CC_PT_VENDITAPRODOTTI')
      AND EXISTS (SELECT 1 FROM AccountantKeysCategories WHERE AccountantKeyCategoryId = 0)


-- Enable Packing Lists for all customers -- the category row does not exist on initializations of databases
IF EXISTS (SELECT * FROM SecuritySystemPermissionsCategories WHERE CategoryId = 3)
	INSERT INTO SecuritySystemPermissions (OID, Name, Description, CategoryId)
	SELECT NEWID(), 'navBarItemPackingLists', 'Goods Flow --> Packing Lists', 3
	WHERE NOT EXISTS (
		SELECT 1
		FROM SecuritySystemPermissions
		WHERE Name = 'navBarItemPackingLists'
	);

	
IF EXISTS(SELECT * FROM SecuritySystemPermissionsCategories WHERE CategoryId = 1)
	INSERT INTO SecuritySystemPermissions(
	OID, Name, Description, CategoryId)
	SELECT NEWID(), 'CanModifyPrescription', 'Client -> Prescriptions -> Modify', 1
	WHERE NOT EXISTS (
		SELECT 1 FROM SecuritySystemPermissions
		WHERE Name = 'CanModifyPrescription'
	);

	

IF EXISTS(SELECT * FROM StoreSetup WHERE ParameterName = 'ShowDiscountOnLabel')
	UPDATE StoreSetup SET ParameterDescription =
	'Specifies if the discount is shown on the labels (True/False/Custom). Custom = Allows user to choose on label settings.' WHERE ParameterName = 'ShowDiscountOnLabel'
	

--Set this to 1 because when the column is created is set to DbNull
Update PosPaymentTypes set IsVoucherSaleEnabled=1 where IsVoucherSaleEnabled is NULL
Update LabelTypes set IsEnabled=1 where IsEnabled is null
Update PosPaymentTypes set IsVoucherSaleEnabled=1 where IsVoucherSaleEnabled is NULL

IF NOT EXISTS (SELECT * FROM SecuritySystemPermissions WHERE Name = 'CanPerformSalesWithoutStoreDayClose') AND EXISTS(SELECT * FROM SecuritySystemPermissionsCategories WHERE CategoryId = 2)
BEGIN
	INSERT INTO SecuritySystemPermissions (OID,Name, Description,CategoryId)
		VALUES (NEWID(), 'CanPerformSalesWithoutStoreDayClose', 'Can perform sales without a store day close being made the previous day',2)

	INSERT INTO SecuritySystemRole_Permissions
		(OID, PermissionId, RoleId)
		SELECT
		NEWID(), (SELECT TOP 1 OID FROM SecuritySystemPermissions WHERE Name = 'CanPerformSalesWithoutStoreDayClose'), OID
	FROM SecuritySystemRole
END
GO
